#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import util, registry, common

ACCELERATION_NAMES = {
    "default": "Full",
    "full": "Full",
    "standard": "Standard",
    "basic": "Basic",
    "emulation": "Emulation"
}
ACCELERATIONS = ACCELERATION_NAMES.keys()
RATES = [48000, 44100, 22050, 16000, 11025, 8000]
BITS = [8, 16]
DRIVER_NAMES = {
    'alsa':      'alsa',
    'oss':       'oss',
    'coreaudio': 'coraudio',
    'jack':      'jack',
    'nas':       'nas',
    'esd':       'esd',
    'audioio':   'audioio',
    'pulse':     'pulse'
}

def get_drivers():
    paths = [
        '{vineyard_path}/lib'.format(vineyard_path = common.ENV['VINEYARDPATH']),
        '/usr/lib',
        '/usr/lib32',
        '/usr/local/lib',
        '/usr/local/lib32'
    ]
    drivers_1 = util.get_command_output("grep -l -e 'winmm' %s" %
        ' '.join('%s/wine/wine*.drv.so' % i for i in paths), shell=True, dont_parse_command=True)
    drivers_2 = util.get_command_output("grep -l -e 'wave' %s" %
        ' '.join('%s/wine/wine*.drv.so' % i for i in paths), shell=True, dont_parse_command=True)
    return list(set(drivers_1.split('\n')) & set(drivers_2.split('\n')))

def get_driver_name(driverfile):
    return driverfile.split('/wine')[-1].split('.drv')[0]

DRIVERS = [ get_driver_name(driver) for driver in get_drivers() ]

def get_acceleration(program=None):
    #try:
    #    return CACHE['audio-acceleration']
    #except KeyError:
    if program:
        level = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % \
            program, "HardwareAcceleration", quiet=True)
    else:
        level = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound',
            "HardwareAcceleration", quiet=True)
    if level:
        #CACHE['audio-acceleration'] = level
        return level
    else:
        #CACHE['audio-acceleration'] = ACCELERATION_NAMES['default']
        return ACCELERATION_NAMES['default']

def set_acceleration(accel, program=None):
    """
        Sets the level of audio acceleration to use.
        accel can be one of ["full", "standard", "basic", "emulation"]
    """
    if program:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % \
            program: {"HardwareAcceleration": ACCELERATION_NAMES[accel]}})
    else:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {
            "HardwareAcceleration": ACCELERATION_NAMES[accel]}})
        #CACHE['audio-acceleration'] = ACCELERATION_NAMES[accel]

def get_sample_rate(program=None):
    if program:
        rate = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program,
            "DefaultSampleRate", quiet=True)
    else:
        #try:
        #    return CACHE['audio-samplerate']
        #except KeyError:
        rate = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound',
            "DefaultSampleRate", quiet=True)
    if rate:
        #if program == None:
        #    CACHE['audio-samplerate'] = rate
        return rate
    else:
        if program == None:
            CACHE['audio-samplerate'] = '44100'
        return '44100'

def set_sample_rate(rate, program=None):
    """
        Sets the DirectSound audio sample rate
    """
    if program:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {
            "DefaultSampleRate": rate}})
    else:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {
            "DefaultSampleRate": rate}})
        #CACHE['audio-samplerate'] = str(rate)

def get_bit_depth(program=None):
    if program:
        bits = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % \
            program, "DefaultBitsPerSample", quiet=True)
    else:
        #try:
        #    return CACHE['audio-bitdepth']
        #except KeyError:
        bits = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound',
            "DefaultBitsPerSample", quiet=True)
    if bits:
        #if program == None:
        #    CACHE['audio-bitdepth'] = bits
        return bits
    else:
        #if program == None:
        #    CACHE['audio-bitdepth'] = '16'
        return '16'

def set_bit_depth(bits, program=None):
    """
        Sets the DirectSound audio bit depth
    """
    if program:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {
            "DefaultBitsPerSample": bits}})
    else:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {
            "DefaultBitsPerSample": bits}})
        #CACHE['audio-bitdepth'] = str(bits)

def get_driver_emulation(program=None):
    if program:
        state = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program,
            "EmulDriver", quiet=True)
    else:
        #try:
        #    return CACHE['audio-emulate-driver']
        #except KeyError:
        state = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound',
            "EmulDriver", quiet=True)
    if state and "y" in state.lower():
        #if program == None:
        #    CACHE['audio-emulate-driver'] = True
        return True
    else:
        #if program == None:
        #    CACHE['audio-emulate-driver'] = False
        return False

def set_driver_emulation(state, program=None):
    """
        Sets wether DirectSound should emulate a hardware driver
    """
    if state:
        state_str = "Y"
    else:
        state_str = "N"
    
    if program:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {
            "EmulDriver": state_str}})
    else:
        registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {
            "EmulDriver": state_str}})
        #CACHE['audio-emulate-driver'] = state

def get_enabled_drivers():
    #try:
    #    return CACHE['audio-enabled-drivers']
    #except KeyError:
    drivers = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Drivers', "Audio", quiet=True)
    if drivers not in [ None, {} ]:
        drivers = drivers.split(',')
        #CACHE['audio-enabled-drivers'] = drivers
        return drivers
    else:
        #CACHE['audio-enabled-drivers'] = []
        return []

def set_enabled_drivers(drivers):
    """
        Sets the list of enabled audio drivers
        disabling all others
    """
    #CACHE['audio-enabled-drivers'] = drivers
    registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Drivers': {"Audio": ','.join(drivers)}})

def enable_driver(driver):
    """
        Adds an audio driver to the list of enabled audio drivers
        driver can be one of ["alsa", "oss", "coreaudio", "jack", "nas", "esd", "audioio"]
    """
    drivers = filter(len, get_enabled_drivers())
    print driver, DRIVER_NAMES
    if driver in DRIVER_NAMES and DRIVER_NAMES[driver] not in drivers:
        drivers.append(DRIVER_NAMES[driver])
        registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Drivers': {"Audio": ','.join(drivers)}})
    else:
        return False

def disable_driver(driver):
    """
        Removes an audio driver from the list of enabled audio drivers
        driver can be one of ["alsa", "oss", "coreaudio", "jack", "nas", "esd", "audioio"]
    """
    drivers = get_enabled_drivers()
    if driver in DRIVER_NAMES and DRIVER_NAMES[driver] in drivers:
        drivers.remove(DRIVER_NAMES[driver])
        registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Drivers': {"Audio": ','.join(drivers)}})
