#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import appearance, shellfolders, util, common, drives, base
import os, subprocess, string, random, time

import logging

logger = logging.getLogger("python-wine.prefixes")
debug = logger.debug
error = logger.error

# FIXME: We need multi-path support, and the default really should be changed
BASEDIR = os.path.expanduser("~/.winebottles")

def list(includedir=False):
    if not os.path.isdir(BASEDIR):
        os.mkdir(BASEDIR)
        return []
    prefixes = os.listdir(BASEDIR)
    if includedir:
        bottlenames = {}
    else:
        bottlenames = []
    for bottle in prefixes:
        if get_type(os.path.join(BASEDIR, bottle)) != 'wine':
            bottlename = get_name(bottle)
            if includedir:
                bottlenames[bottlename] = bottle
            else:
                bottlenames.append(bottlename)
    return bottlenames

def add(bottlename):
    if bottlename is None or not len(bottlename.strip()):
        raise ValueError, bottlename
    HOME = util.getRealHome()
    bottledir = get_dir_name(bottlename)
    if bottledir:
        path = "%s/%s" % (BASEDIR, bottledir)
    else:
        if not os.path.isdir(BASEDIR):
            os.mkdir(BASEDIR)
        bottledir = create_dir_name(bottlename)
        path = "%s/%s" % (BASEDIR, bottledir)
        if not os.path.isdir(path):
            os.mkdir(path)
        path_xdg = os.path.join(path, 'xdg')
        if not os.path.isdir(path_xdg):
            os.mkdir(path_xdg)
            os.mkdir(os.path.join(path_xdg, 'local'))
            os.mkdir(os.path.join(path_xdg, 'local', 'share'))
        path_vineyard_data = os.path.join(path, 'vineyard')
        if not os.path.isdir(path_vineyard_data):
            os.mkdir(path_vineyard_data)
        with open(os.path.join(path_vineyard_data, 'name'), 'w') as namefile:
            namefile.write(bottlename)

    #common.ENV['HOME'] = path
    #common.ENV['WINEPREFIX'] = "%s/.wine" % path
    common.ENV['WINEPREFIX'] = path

    returncode = wine_first_run()

    # Add a link to the main drive in its directory
    for drive_letter in ['c', 'C']:
        drive_path = '%s/dosdevices/%s:' % (common.ENV['WINEPREFIX'], drive_letter)
        if os.path.exists(drive_path):
            drive_symlink = '%s/main drive' % path
            if os.path.lexists(drive_symlink):
                os.remove(drive_symlink)
            os.symlink(drive_path, drive_symlink)
            break

    # Use the added prefix
    use(get_name(path))

    return returncode == 0

def remove(name=None, dirname=None):
    HOME = util.getRealHome()
    if name is None:
        path = "%s/%s" % (BASEDIR, dirname)
    else:
        path = "%s/%s" % (BASEDIR, get_dir_name(name))

    if os.path.isdir(path):
        process = common.Popen(["rm", "-rf", path], env=common.ENV).wait()
        return process == 0
    else:
        error("Bottle doesn't exist.")
        return True

def use(bottlename):
    HOME = util.getRealHome()
    common.ENV['HOME'] = "%s" % HOME # just to make sure it's there
    if bottlename == None:
        common.ENV['WINEPREFIX'] = "%s/.wine" % HOME
        common.ENV['VINEYARDCONF'] = ''
        common.ENV['VINEYARDCONFNAME'] = ''

        common.ENV['XDG_CONFIG_HOME'] = common.ENV.get(
            'XDG_CONFIG_HOME', os.path.join(
                HOME, '.config'
            )
        )
        common.ENV['XDG_DATA_HOME'] = common.ENV.get(
            'XDG_DATA_HOME', os.path.join(
                HOME, '.local', 'share'
            )
        )
        common.ENV['XDG_DATA_DIRS'] = common.ENV.get(
            'XDG_DATA_DIRS', os.path.join(
                HOME, '.local', 'share'
            )
        )
        # If there is no default Wine prefix, create it
        if not base.check_setup():
            wine_first_run()
    else:
        bottledir = get_dir_name(bottlename)
        if bottledir:
            common.ENV['VINEYARDCONF'] = bottledir
            common.ENV['VINEYARDCONFNAME'] = get_name(bottledir)
            # If this is a unified style prefix
            if get_type(os.path.join(BASEDIR, bottledir)) == 'unified':
                common.ENV['WINEPREFIX'] = os.path.join(BASEDIR, bottledir)
                common.ENV['XDG_CONFIG_HOME'] = os.path.join(
                    BASEDIR, bottledir, 'xdg', 'config'
                )
                common.ENV['XDG_DATA_HOME'] = os.path.join(
                    BASEDIR, bottledir, 'xdg', 'local', 'share'
                )
                common.ENV['XDG_DATA_DIRS'] = os.path.join(
                    BASEDIR, bottledir, 'xdg', 'local', 'share'
                )
            # Legacy Vineyard configuration
            else:
                common.ENV['WINEPREFIX'] = "%s/%s/.wine" % (BASEDIR, bottledir)
                common.ENV['XDG_CONFIG_HOME'] = os.path.join(
                    BASEDIR, bottledir, '.config'
                )
                common.ENV['XDG_DATA_HOME'] = os.path.join(
                    BASEDIR, bottledir, '.local', 'share'
                )
                common.ENV['XDG_DATA_DIRS'] = os.path.join(
                    BASEDIR, bottledir, '.local', 'share'
                )
    common.ENV['VINEYARD_DATA'] = os.path.join(
        common.ENV['WINEPREFIX'], 'vineyard'
    )

    #setup_variables()

    if bottlename == None or get_dir_name(bottlename):
        drives.auto_mount_images()
        return True
    else:
        return False


def wine_first_run():
    # Create Wine structure (in place of wineprefixcreate, since it's depricated)
    process = common.Popen(
        ['regedit', '/E', '/tmp/.winebottlecreated', 'HKEY_CURRENT_USER\\Software\\Wine'],
        stdout='null', stderr='null', env=common.ENV_NO_DISPLAY())
    process.wait()
    # Regedit isn't always actually finished when it says it is, so check it first
    while base.check_setup() is False:
        time.sleep(0.2)
    appearance.set_menu_style(True)
    shellfolders.setdefaults()
    return process.returncode


def get_current_name():
    return get_name(get_current_dir_name())

def get_current_dir_name():
    if os.path.exists(os.path.join(common.ENV['WINEPREFIX'], 'vineyard')):
        return os.path.basename(common.ENV['WINEPREFIX'])
    else:
        return os.path.basename(os.path.dirname(common.ENV['WINEPREFIX']))

def create_dir_name(name=None):
    if name is None:
        name = util.string_random(length=8)
    else:
        # Make name safe for use on any filesystem (use only alphanumeric chars)
        name = util.string_safe_chars(name, remove_repeats=True).lower()

    dirname = name[:]

    if os.path.exists("%s/%s" % (BASEDIR, dirname)):
        nr = 1
        while os.path.exists("%s/%s" % (BASEDIR, dirname)):
            dirname = '{name}-{nr}'.format(
                name = name,
                nr = nr
            )

    return dirname

def get_dir_name(bottlename):
    """Get the directory name of a configuration based on its name."""
    prefixes = util.dict_to_case_insensitive(
        list(True)
    )
    #for key in prefixes.keys():
    #    prefixes[key.lower()] = prefixes[key]

    bottlename = bottlename.lower().strip()

    if bottlename in prefixes:
        return prefixes[bottlename]
    else:
        if bottlename.endswith('*'):
            matches = []
            for name, prefix in prefixes.iteritems():
                if name.startswith(bottlename.split('*')[0]):
                    matches.append(prefix)
            if len(matches):
                return matches[0]
        return False

def get_dir(bottlename):
    """Get the full directory of a configuration based on its name."""
    bottle_dir = get_dir_name(bottlename)
    return "%s/%s" % (BASEDIR, bottle_dir)

def _get_namefile(prefix_dir):
    if os.path.isdir(prefix_dir):
        if get_type(prefix_dir) == 'legacy':
            return os.path.join(prefix_dir, '.name')
        else:
            if not os.path.exists(os.path.join(prefix_dir, 'vineyard')):
                os.mkdir(os.path.join(prefix_dir, 'vineyard'))
            return os.path.join(prefix_dir, 'vineyard', 'name')
    return None

def get_name(dirname):
    """Get the name of a configuration based on its directory name."""
    namefile = _get_namefile(os.path.join(BASEDIR, dirname))
    if namefile is None:
        return None

    try:
        with open(namefile, 'r') as namefile:
            bottlename = namefile.read().strip()
        return bottlename
    except IOError:
        return None

def set_name(dirname, bottlename):
    """Set the name of a configuration."""
    if os.path.isdir(os.path.join(BASEDIR, dirname, '.wine')):
        namefile = os.path.join(BASEDIR, dirname, '.name')
    else:
        namefile = os.path.join(BASEDIR, dirname, 'vineyard', 'name')

    try:
        with open(namefile, 'w') as namefile:
            namefile.write(bottlename.strip())
        return bottlename
    except IOError:
        return get_name(dirname)

def set_current_name(name):
    dirname = os.path.basename(os.path.dirname(common.ENV['WINEPREFIX']))
    return set_name(dirname, name)

def get_name_from_path(path):
    """Try to get the name of the configuration this path is in, if any. Returns None if no configuration found."""
    paths_and_names = [
        (os.path.join(BASEDIR, i[1]), i[0])
        for i in list(True).iteritems()
    ]
    for _path, name in paths_and_names:
        if path.startswith(_path):
            return name
    return None

def get_type(prefix_path):
    """Return type of prefix. Returns either 'default', 'unified', 'legacy' or 'wine'."""
    if (
        prefix_path == os.path.expanduser('~/.wine')
    ):
        return 'default'
    elif (
        os.path.exists(os.path.join(prefix_path, 'xdg')) and
        os.path.exists(os.path.join(prefix_path, 'system.reg'))
    ):
        return 'unified'
    elif (
        os.path.exists(os.path.join(prefix_path, '.name')) and
        os.path.exists(os.path.join(prefix_path, '.wine'))
    ):
        return 'legacy'
    elif (
        os.path.exists(os.path.join(prefix_path, 'dosdevices')) and
        os.path.exists(os.path.join(prefix_path, 'system.reg'))
    ):
        return 'wine'

def reboot():
    process = common.Popen(
        ['wineboot', '--restart'],
        env=common.ENV_NO_DISPLAY()
    )
    process.wait()

def shutdown():
    process = common.Popen(
        ['wineboot', '--shutdown'],
        env=common.ENV_NO_DISPLAY()
    )
    process.wait()

def end_session(force=False):
    if force:
        process = common.Popen(
            ['wineboot', '--end-session', '--force'],
            env=common.ENV_NO_DISPLAY()
        )
    else:
        process = common.Popen(
            ['wineboot', '--end-session'],
            env=common.ENV_NO_DISPLAY()
        )
    process.wait()

def kill():
    process = common.Popen(
        ['wineboot', '--kill'],
        env=common.ENV_NO_DISPLAY()
    )
    process.wait()


def setup_variables():
    """Set up mostly the same variables that wisotool uses for scripts - they are handy"""
    common.ENV['W_DRIVE_C'] = drives.get_main_drive(use_registry=False)['mapping']
    common.ENV['W_PREFIXES_ROOT'] = BASEDIR
    # Verbs can rely on W_TMP being empty at entry, deleted after return, and a subdir of C:
    # Kludge: use _temp instead of temp to avoid \t expansion in w_try
    common.ENV['W_TMP'] = "{0[W_DRIVE_C]}/windows/_temp".format(common.ENV)
    common.ENV['W_TMP_WIN'] = "C:\\windows\\_temp"

    common.ENV['W_WINDIR_UNIX'] = "{0[W_DRIVE_C]}/windows".format(common.ENV)

    # The folder-name is localized!
    common.ENV['W_PROGRAMS_WIN'] = util.get_win_env_path('ProgramFiles')
    if common.ENV['W_PROGRAMS_WIN'] == None:
        common.ENV['W_PROGRAMS_WIN'] = "{0[W_DRIVE_C]}/Program Files".format(common.ENV)
    common.ENV['W_PROGRAMS_UNIX'] = util.wintounix(common.ENV['W_PROGRAMS_WIN'])

    # 64 bit windows has a second directory for program files
    common.ENV['W_PROGRAMS_X86_WIN'] = "{0[W_PROGRAMS_WIN]} (x86)".format(common.ENV)
    common.ENV['W_PROGRAMS_X86_UNIX'] = "{0[W_PROGRAMS_UNIX]} (x86)".format(common.ENV)
    if not os.path.isdir(common.ENV['W_PROGRAMS_X86_UNIX']):
        common.ENV['W_PROGRAMS_X86_WIN'] = common.ENV['W_PROGRAMS_WIN']
        common.ENV['W_PROGRAMS_X86_UNIX'] = common.ENV['W_PROGRAMS_UNIX']

    common.ENV['W_APPDATA_WIN'] = util.get_win_env_path('AppData')
    if common.ENV['W_APPDATA_WIN'] == None:
        common.ENV['W_APPDATA_WIN'] = "{0[W_DRIVE_C]}/Application Data".format(common.ENV)
    common.ENV['W_APPDATA_UNIX'] = util.wintounix(common.ENV['W_APPDATA_WIN'])

    # FIXME: get fonts path from SHGetFolderPath
    # See also http://blogs.msdn.com/oldnewthing/archive/2003/11/03/55532.aspx
    common.ENV['W_FONTSDIR_WIN'] = "c:\\windows\\Fonts"
    common.ENV['W_FONTSDIR_UNIX'] = util.wintounix(common.ENV['W_FONTSDIR_WIN'])
    if not os.path.isdir(common.ENV['W_FONTSDIR_UNIX']):
        os.mkdir(common.ENV['W_FONTSDIR_UNIX'])

    # FIXME: don't hardcode
    common.ENV['W_PROGRAMS_DRIVE'] = common.ENV['W_DRIVE_C']

    # Win(e) 32/64?
    # Using the variable W_SYSTEM32_DLLS instead of SYSTEM32 because some stuff does go under system32 for both arch's
    # e.g., spool/drivers/color
    if os.path.isdir("{0[W_DRIVE_C]}/windows/syswow64".format(common.ENV)):
        common.ENV['W_ARCH'] = 'win64'
        common.ENV['W_SYSTEM32_DLLS'] = "{0[W_WINDIR_UNIX]}/syswow64".format(common.ENV)
        common.ENV['W_SYSTEM32_DLLS_WIN'] = "C:\\windows\\syswow64"
        common.ENV['W_SYSTEM64_DLLS'] = "{0[W_WINDIR_UNIX]}/system32".format(common.ENV)
    else:
        common.ENV['W_ARCH'] = 'win32'
        common.ENV['W_SYSTEM32_DLLS'] = "{0[W_WINDIR_UNIX]}/system32".format(common.ENV)
        common.ENV['W_SYSTEM32_DLLS_WIN'] = "C:\\windows\\system32"

