#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2010 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import common, registry

# wine/programs/winecfg/appdefaults.c
DEFAULT = 'winxp'
windowsversions_sorted = common.sorteddict(
    ("win7", [
        "Windows 7", 6,1,
        7601, "WIN32_NT", "Service Pack 1", 0, 0, "WinNT"
    ]),
    ("win2008", [
        "Windows 2008", 6,0,
        6001, "WIN32_NT", "Service Pack 1", 0, 0, "ServerNT"
    ]),
    ("vista", [
        "Windows Vista", 6,0,
        6000, "WIN32_NT", "Service Pack 2", 2, 0, "WinNT"
    ]),
    ("win2003", [
        "Windows 2003", 5,2,
        3790, "WIN32_NT", "Service Pack 1", 1, 0, "ServerNT"
    ]),
    ("winxp", [
        "Windows XP", 5,1,
        2600, "WIN32_NT", "Service Pack 3", 2, 0, "WinNT"
    ]),
    ("win2k", [
        "Windows 2000", 5,0,
        2195, "WIN32_NT", "Service Pack 4", 4, 0, "WinNT"
    ]),
    ("winme", [
        "Windows ME", 4,90,
        3000, "WIN32_WINDOWS", " ", 0, 0, ""
    ]),
    ("win98", [
        "Windows 98", 4,10,
        2222, "WIN32_WINDOWS", " A ", 0, 0, ""
    ]),
    ("win95", [
        "Windows 95", 4,0,
        950, "WIN32_WINDOWS", "", 0, 0, ""
    ]),
    ("nt40", [
        "Windows NT 4.0", 4,0,
        1381, "WIN32_NT", "Service Pack 6a", 6, 0, "WinNT"
    ]),
    ("nt351", [
        "Windows NT 3.5", 3,51,
        1057, "WIN32_NT", "Service Pack 2", 0, 0, "WinNT"
    ]),
    ("win31", [
        "Windows 3.1", 2,10,
        0, "WIN32s", "Win32s 1.3", 0, 0, ""
    ]),
    ("win30", [
        "Windows 3.0", 3,0,
        0, "WIN32s", "Win32s 1.3", 0, 0, ""
    ]),
    ("win20", [
        "Windows 2.0", 2,0,
        0, "WIN32s", "Win32s 1.3", 0, 0, ""
    ])
)
windowsversions = windowsversions_sorted.dict()
versions = [ i for i in reversed(windowsversions_sorted.keys()) ]
wDescription = 0
wMajorVersion = 1
wMinorVersion = 2
wBuildNumber = 3
wPlatformId = 4
wCSDVersion = 5
wServicePackMajor = 6
wServicePackMinor = 7
wProductType = 8
Key9x = "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion"
KeyNT = "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion"
KeyProdNT = "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\ProductOptions"
KeyWindNT = "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Windows"
KeyEnvNT  = "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Session Manager\\Environment"

def get(program=None):
    if program:
        version = None
        programs = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults', quiet=True)
        for key in programs.keys():
            programs[key.lower()] = programs[key]
        if program.lower() in programs.keys() and 'Version' in programs[program.lower()]:
            version = programs[program.lower()]['Version']
        if version == None or version not in windowsversions.keys():
            return DEFAULT
        else:
            return version
    else:
        version = registry.get('HKEY_CURRENT_USER\\Software\\Wine', "Version", quiet=True)
        if version:
            if version == None or version not in windowsversions.keys():
                return DEFAULT
            else:
                return version
        else:
            return _parseWineVersionFromRegistry()

def set(version, program=False):
    version = __translate_version_string(version)
    if version not in versions:
        raise KeyError(
            "Error: Version should be one of the following: default, {0}".format(
                ", ".join(versions)
            )
        )
    
    if program:
        registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults\\%s' % program: {"Version": version} })
    else:
        set_default(version)

def set_default(version):
    version = __translate_version_string(version, strict=True)
    shortversion = version
    version = windowsversions[version]
          
    versionnumber9x = "%s.%s.%s" % (version[wMajorVersion], version[wMinorVersion], version[wBuildNumber])
    versionnumberNT = "%s.%s" % (version[wMajorVersion], version[wMinorVersion])
    
    if version[wPlatformId] == "WIN32_WINDOWS":
        registry_values = {
            Key9x: {
                "VersionNumber": versionnumber9x,
                "SubVersionNumber": version[wCSDVersion]
            },
            KeyNT: { "CSDVersion": None, "CurrentVersion": None, "CurrentBuildNumber": None },
            KeyProdNT: { "ProductType": None },
            KeyWindNT: { "CSDVersion": None },
            KeyEnvNT: { "OS": None },
            'HKEY_CURRENT_USER\\Software\\Wine': {"Version": None}
        }
    elif version[wPlatformId] == "WIN32_NT":
        registry_values = {
            KeyNT: {
                "CurrentVersion": versionnumberNT,
                "CSDVersion": version[wCSDVersion],
                "CurrentBuildNumber": version[wBuildNumber]
            },
            KeyProdNT: { "ProductType": version[wProductType] },
            KeyWindNT: { "CSDVersion": "dword:00000%s00" % version[wServicePackMajor] },
            KeyEnvNT: { "OS": "Windows_NT" },
            Key9x: { "VersionNumber": None, "SubVersionNumber": None },
            'HKEY_CURRENT_USER\\Software\\Wine': {"Version": None}
        }
    elif version[wPlatformId] == "WIN32s":
        registry_values = {
            KeyNT: { "CSDVersion": None, "CurrentVersion": None, "CurrentBuildNumber": None },
            KeyProdNT: { "ProductType": None },
            KeyWindNT: { "CSDVersion": None },
            KeyEnvNT: { "OS": None },
            Key9x: { "VersionNumber": None, "SubVersionNumber": None },
            'HKEY_CURRENT_USER\\Software\\Wine': {"Version": shortversion}
        }
    registry.set(registry_values)

def _parseWineVersionFromRegistry():
    # Note, good info here as well, though it's a bit too precise for what we
    # need for Wine... but you never know ;)
    # http://techsupt.winbatch.com/TS/T000001074F4.html
    versionnt = registry.get(KeyNT, "CurrentVersion")
    version9x = registry.get(Key9x, "VersionNumber")
    best = DEFAULT # hardcoded fallback (same as in winecfg)
    
    if versionnt:
        version = versionnt
        platform = "WIN32_NT"
        build = int(registry.get(KeyNT, "CurrentBuildNumber"))
    elif version9x:
        version = version9x
        platform = "WIN32_WINDOWS"
    else:
        return best
    
    # Strip non-legal characters, if needed
    try:
        int(version)
    except ValueError:
        chars = '0123456789.'
        version = ''.join(( i for i in version if i in chars ))
    
    if '.' in version:
        parts = version.split('.')
        if len(parts) == 3:
            build = int(parts[2])
        minor = int(parts[1])
        major = int(parts[0])
    else:
        major = int(version)
    
    for winver,data in windowsversions.iteritems():
        if data[wPlatformId] != platform or data[wMajorVersion] != major:
            continue
        best = winver
        if data[wMinorVersion] == minor and data[wBuildNumber] == build:
            return winver
    return best

def get_programs():
    programs = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults', quiet=True)
    for key in programs.keys():
        programs[key.lower()] = programs[key]
    return programs

def __translate_version_string(version, strict=False):
    version = version.lower()
    if version == "default":
        return DEFAULT
    elif version == "win2000":
        return "win2k"
    return version
